#![feature(iter_array_chunks)]
#![feature(is_some_and)]
use std::cmp::Ordering;

fn main() {
    let json = std::fs::read_to_string("/dev/stdin").unwrap();
    let mut p1_tot = 0;
    let mut p2_tot = 1;

    let starting_packet: serde_json::Value = serde_json::from_str("[[2]]").unwrap();
    let ending_packet: serde_json::Value = serde_json::from_str("[[6]]").unwrap();

    for (i, [left, right]) in json
        .lines()
        .filter(|l| !l.is_empty())
        .array_chunks()
        .enumerate()
    {
        let left: serde_json::Value = serde_json::from_str(left).unwrap();
        let right: serde_json::Value = serde_json::from_str(right).unwrap();

        let right_order = serde_cmp(Some(&left), Some(&right)).1;
        if right_order {
            p1_tot += i + 1;
        }
    }

    let mut all = json
        .lines()
        .filter(|l| !l.is_empty())
        .map(|l| serde_json::from_str::<serde_json::Value>(l).unwrap())
        .collect::<Vec<_>>();

    all.push(starting_packet.clone());
    all.push(ending_packet.clone());

    all.sort_by(|a, b| match serde_cmp(Some(a), Some(b)).1 {
        true => Ordering::Less,
        false => Ordering::Greater,
    });

    for (idx, line) in all.iter().enumerate() {
        if *line == starting_packet || *line == ending_packet {
            p2_tot *= idx + 1;
        }
    }

    println!("P1: {}, P2: {}", p1_tot, p2_tot);
}

// cmp returns two bools: keep checking?, and current values in right order?
fn serde_cmp(l: Option<&serde_json::Value>, r: Option<&serde_json::Value>) -> (bool, bool) {
    if l.is_none() && r.is_none() {
        panic!("both values are none");
    }

    match (l, r) {
        (Some(_), None) => return (false, false),
        (None, Some(_)) => return (false, true),
        (None, None) => return (true, false),
        (_, _) => (),
    }

    let (l, r) = (l.unwrap(), r.unwrap());

    if l.is_number() && r.is_number() {
        let (ln, rn) = (l.as_i64().unwrap(), r.as_i64().unwrap());

        match ln.cmp(&rn) {
            Ordering::Greater => return (false, false),
            Ordering::Equal => return (true, false),
            Ordering::Less => return (false, true),
        }
    }

    let (fl, fr) = (to_flat_vec(l), to_flat_vec(r));
    for i in 0..std::cmp::max(fl.len(), fr.len()) {
        let (more, res) = serde_cmp(fl.get(i).copied(), fr.get(i).copied());
        if !more {
            return (more, res);
        }
    }

    (true, true)
}

fn to_flat_vec(val: &serde_json::Value) -> Vec<&serde_json::Value> {
    if val.is_number() {
        return vec![val];
    }

    if val.is_array() {
        return val.as_array().unwrap().iter().collect::<Vec<_>>();
    }

    panic!("val is neither array or num")
}
